@@ -13,7 +13,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkText.c 1.104 97/10/13 15:18:24
+ * RCS: @(#) $Id: tkText.c,v 1.2 1998/09/14 18:23:17 stanton Exp $
  */
 
 #include "default.h"
@@ -27,28 +27,6 @@
 
 #include "tkText.h"
 
-/*
- * Custom options for handling "-state" and "-tile"
- */
-
-static Tk_CustomOption stateOption = {
-    (Tk_OptionParseProc *) Tk_StateParseProc,
-    Tk_StatePrintProc,
-    (ClientData) NULL	/* only "normal" and "disabled" */
-};
-
-static Tk_CustomOption tileOption = {
-    (Tk_OptionParseProc *) Tk_TileParseProc,
-    Tk_TilePrintProc,
-    (ClientData) NULL
-};
-
-static Tk_CustomOption offsetOption = {
-    (Tk_OptionParseProc *) Tk_OffsetParseProc,
-    Tk_OffsetPrintProc,
-    (ClientData) NULL
-};
-
 /*
  * Information used to parse text configuration options:
  */
@@ -66,8 +44,6 @@ static Tk_ConfigSpec configSpecs[] = {
 	DEF_TEXT_BORDER_WIDTH, Tk_Offset(TkText, borderWidth), 0},
     {TK_CONFIG_ACTIVE_CURSOR, "-cursor", "cursor", "Cursor",
 	DEF_TEXT_CURSOR, Tk_Offset(TkText, cursor), TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-disabledtile", "disabledtile", "Tile", (char *) NULL,
-	Tk_Offset(TkText, disabledTile), TK_CONFIG_DONT_SET_DEFAULT, &tileOption},
     {TK_CONFIG_BOOLEAN, "-exportselection", "exportSelection",
 	"ExportSelection", DEF_TEXT_EXPORT_SELECTION,
 	Tk_Offset(TkText, exportSelection), 0},
@@ -101,9 +77,6 @@ static Tk_ConfigSpec configSpecs[] = {
 	DEF_TEXT_INSERT_ON_TIME, Tk_Offset(TkText, insertOnTime), 0},
     {TK_CONFIG_PIXELS, "-insertwidth", "insertWidth", "InsertWidth",
 	DEF_TEXT_INSERT_WIDTH, Tk_Offset(TkText, insertWidth), 0},
-    {TK_CONFIG_CUSTOM, "-offset", "offset", "Offset", "0,0",
-	Tk_Offset(TkText, tsoffset), TK_CONFIG_DONT_SET_DEFAULT,
-	&offsetOption},
     {TK_CONFIG_PIXELS, "-padx", "padX", "Pad",
 	DEF_TEXT_PADX, Tk_Offset(TkText, padX), 0},
     {TK_CONFIG_PIXELS, "-pady", "padY", "Pad",
@@ -139,19 +112,17 @@ static Tk_ConfigSpec configSpecs[] = {
     {TK_CONFIG_PIXELS, "-spacing3", "spacing3", "Spacing",
 	DEF_TEXT_SPACING3, Tk_Offset(TkText, spacing3),
 	TK_CONFIG_DONT_SET_DEFAULT},
-    {TK_CONFIG_CUSTOM, "-state", "state", "State",
-	DEF_TEXT_STATE, Tk_Offset(TkText, state), 0, &stateOption},
+    {TK_CONFIG_UID, "-state", "state", "State",
+	DEF_TEXT_STATE, Tk_Offset(TkText, state), 0},
     {TK_CONFIG_STRING, "-tabs", "tabs", "Tabs",
 	DEF_TEXT_TABS, Tk_Offset(TkText, tabOptionString), TK_CONFIG_NULL_OK},
     {TK_CONFIG_STRING, "-takefocus", "takeFocus", "TakeFocus",
 	DEF_TEXT_TAKE_FOCUS, Tk_Offset(TkText, takeFocus),
 	TK_CONFIG_NULL_OK},
-    {TK_CONFIG_CUSTOM, "-tile", "tile", "Tile", (char *) NULL,
-	Tk_Offset(TkText, tile),TK_CONFIG_DONT_SET_DEFAULT, &tileOption},
     {TK_CONFIG_INT, "-width", "width", "Width",
 	DEF_TEXT_WIDTH, Tk_Offset(TkText, width), 0},
-    {TK_CONFIG_CUSTOM, "-wrap", "wrap", "Wrap",
-	DEF_TEXT_WRAP, Tk_Offset(TkText, wrapMode), 0, &textWrapModeOption},
+    {TK_CONFIG_UID, "-wrap", "wrap", "Wrap",
+	DEF_TEXT_WRAP, Tk_Offset(TkText, wrapMode), 0},
     {TK_CONFIG_STRING, "-xscrollcommand", "xScrollCommand", "ScrollCommand",
 	DEF_TEXT_XSCROLL_COMMAND, Tk_Offset(TkText, xScrollCmd),
 	TK_CONFIG_NULL_OK},
@@ -162,6 +133,16 @@ static Tk_ConfigSpec configSpecs[] = {
 	(char *) NULL, 0, 0}
 };
 
+/*
+ * Tk_Uid's used to represent text states:
+ */
+
+Tk_Uid tkTextCharUid = NULL;
+Tk_Uid tkTextDisabledUid = NULL;
+Tk_Uid tkTextNoneUid = NULL;
+Tk_Uid tkTextNormalUid = NULL;
+Tk_Uid tkTextWordUid = NULL;
+
 /*
  * Boolean variable indicating whether or not special debugging code
  * should be executed.
@@ -169,130 +150,6 @@ static Tk_ConfigSpec configSpecs[] = {
 
 int tkTextDebug = 0;
 
-/*
- * Custom options for handling "-wrap " and "-tile":
- */
-
-static int		WrapModeParseProc _ANSI_ARGS_((ClientData clientData,
-			    Tcl_Interp *interp, Tk_Window tkwin, char *value,
-			    char *widgRec, int offset));
-static char *		WrapModePrintProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Window tkwin, char *widgRec, int offset,
-			    Tcl_FreeProc **freeProcPtr));
-
-Tk_CustomOption textWrapModeOption = {
-    WrapModeParseProc,
-    WrapModePrintProc,
-    (ClientData) NULL
-};
-
-
-/*
- *--------------------------------------------------------------
- *
- * WrapModeParseProc --
- *
- *	This procedure is invoked during option processing to handle
- *	"-wrap" options for text widgets.
- *
- * Results:
- *	A standard Tcl return value.
- *
- * Side effects:
- *	The wrap mode for a given item gets replaced by the wrap mode
- *	indicated in the value argument.
- *
- *--------------------------------------------------------------
- */
-
-static int
-WrapModeParseProc(clientData, interp, tkwin, value, widgRec, offset)
-    ClientData clientData;		/* some flags.*/
-    Tcl_Interp *interp;			/* Used for reporting errors. */
-    Tk_Window tkwin;			/* Window containing canvas widget. */
-    char *value;			/* Value of option (list of tag
-					 * names). */
-    char *widgRec;			/* Pointer to record for item. */
-    int offset;				/* Offset into item. */
-{
-    int c;
-    size_t length;
-
-    register TkWrapMode *wrapPtr = (TkWrapMode *) (widgRec + offset);
-
-    if(value == NULL || *value == 0) {
-	*wrapPtr = TEXT_WRAPMODE_NULL;
-	return TCL_OK;
-    }
-
-    c = value[0];
-    length = strlen(value);
-
-    if ((c == 'c') && (strncmp(value, "char", length) == 0)) {
-	*wrapPtr = TEXT_WRAPMODE_CHAR;
-	return TCL_OK;
-    }
-    if ((c == 'n') && (strncmp(value, "none", length) == 0)) {
-	*wrapPtr = TEXT_WRAPMODE_NONE;
-	return TCL_OK;
-    }
-    if ((c == 'w') && (strncmp(value, "word", length) == 0)) {
-	*wrapPtr = TEXT_WRAPMODE_WORD;
-	return TCL_OK;
-    }
-
-    Tcl_AppendResult(interp, "bad wrap mode \"", value,
-	    "\": must be char, none, or word",
-	    (char *) NULL);
-    *wrapPtr = TEXT_WRAPMODE_CHAR;
-    return TCL_ERROR;
-}
-
-/*
- *--------------------------------------------------------------
- *
- * WrapModePrintProc --
- *
- *	This procedure is invoked by the Tk configuration code
- *	to produce a printable string for the "-wrap" configuration
- *	option for canvas items.
- *
- * Results:
- *	The return value is a string describing the state for
- *	the item referred to by "widgRec".  In addition, *freeProcPtr
- *	is filled in with the address of a procedure to call to free
- *	the result string when it's no longer needed (or NULL to
- *	indicate that the string doesn't need to be freed).
- *
- * Side effects:
- *	None.
- *
- *--------------------------------------------------------------
- */
-
-static char *
-WrapModePrintProc(clientData, tkwin, widgRec, offset, freeProcPtr)
-    ClientData clientData;		/* Ignored. */
-    Tk_Window tkwin;			/* Window containing canvas widget. */
-    char *widgRec;			/* Pointer to record for item. */
-    int offset;				/* Ignored. */
-    Tcl_FreeProc **freeProcPtr;		/* Pointer to variable to fill in with
-					 * information about how to reclaim
-					 * storage for return string. */
-{
-    register TkWrapMode *wrapPtr = (TkWrapMode *) (widgRec + offset);
-
-    if (*wrapPtr==TEXT_WRAPMODE_CHAR) {
-	return "char";
-    } else if (*wrapPtr==TEXT_WRAPMODE_NONE) {
-	return "none";
-    } else if (*wrapPtr==TEXT_WRAPMODE_WORD) {
-	return "word";
-    } else {
-	return "";
-    }
-}
-
 /*
  * Forward declarations for procedures defined later in this file:
  */
@@ -325,8 +182,6 @@ static void		DumpLine _ANSI_ARGS_((Tcl_Interp *interp,
 static int		DumpSegment _ANSI_ARGS_((Tcl_Interp *interp, char *key,
 			    char *value, char * command, int lineno, int offset,
 			    int what));
-static void		TileChangedProc _ANSI_ARGS_((ClientData clientData,
-			    Tk_Tile tile, Tk_Item *itemPtr));
 
 /*
  * The structure below defines text class behavior by means of procedures
@@ -376,6 +231,18 @@ Tk_TextCmd(clientData, interp, argc, argv)
 	return TCL_ERROR;
     }
 
+    /*
+     * Perform once-only initialization:
+     */
+
+    if (tkTextNormalUid == NULL) {
+	tkTextCharUid = Tk_GetUid("char");
+	tkTextDisabledUid = Tk_GetUid("disabled");
+	tkTextNoneUid = Tk_GetUid("none");
+	tkTextNormalUid = Tk_GetUid("normal");
+	tkTextWordUid = Tk_GetUid("word");
+    }
+
     /*
      * Create the window.
      */
@@ -398,7 +265,7 @@ Tk_TextCmd(clientData, interp, argc, argv)
     Tcl_InitHashTable(&textPtr->markTable, TCL_STRING_KEYS);
     Tcl_InitHashTable(&textPtr->windowTable, TCL_STRING_KEYS);
     Tcl_InitHashTable(&textPtr->imageTable, TCL_STRING_KEYS);
-    textPtr->state = TK_STATE_NORMAL;
+    textPtr->state = tkTextNormalUid;
     textPtr->border = NULL;
     textPtr->borderWidth = 0;
     textPtr->padX = 0;
@@ -416,7 +283,7 @@ Tk_TextCmd(clientData, interp, argc, argv)
     textPtr->spacing3 = 0;
     textPtr->tabOptionString = NULL;
     textPtr->tabArrayPtr = NULL;
-    textPtr->wrapMode = TEXT_WRAPMODE_CHAR;
+    textPtr->wrapMode = tkTextCharUid;
     textPtr->width = 0;
     textPtr->height = 0;
     textPtr->setGrid = 0;
@@ -449,12 +316,6 @@ Tk_TextCmd(clientData, interp, argc, argv)
     textPtr->xScrollCmd = NULL;
     textPtr->yScrollCmd = NULL;
     textPtr->flags = 0;
-    textPtr->tile = NULL;
-    textPtr->disabledTile = NULL;
-    textPtr->tileGC = None;
-    textPtr->tsoffset.flags = 0;
-    textPtr->tsoffset.xoffset = 0;
-    textPtr->tsoffset.yoffset = 0;
 
     /*
      * Create the "sel" tag and the "current" and "insert" marks.
@@ -636,7 +497,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    result = TCL_ERROR;
 	    goto done;
 	}
-	if (textPtr->state == TK_STATE_NORMAL) {
+	if (textPtr->state == tkTextNormalUid) {
 	    result = DeleteChars(textPtr, argv[2],
 		    (argc == 4) ? argv[3] : (char *) NULL);
 	}
@@ -739,7 +600,7 @@ TextWidgetCmd(clientData, interp, argc, argv)
 	    result = TCL_ERROR;
 	    goto done;
 	}
-	if (textPtr->state == TK_STATE_NORMAL) {
+	if (textPtr->state == tkTextNormalUid) {
 	    for (j = 3;  j < argc; j += 2) {
 		InsertChars(textPtr, &index1, argv[j]);
 		if (argc > (j+1)) {
@@ -859,15 +720,6 @@ DestroyText(memPtr)
     if (textPtr->bindingTable != NULL) {
 	Tk_DeleteBindingTable(textPtr->bindingTable);
     }
-    if (textPtr->tile != NULL) {
-	Tk_FreeTile(textPtr->tile);
-    }
-    if (textPtr->disabledTile != NULL) {
-	Tk_FreeTile(textPtr->disabledTile);
-    }
-    if (textPtr->tileGC != None) {
-	Tk_FreeGC(textPtr->display, textPtr->tileGC);
-    }
 
     /*
      * NOTE: do NOT free up selBorder, selBdString, or selFgColorPtr:
@@ -882,26 +734,6 @@ DestroyText(memPtr)
     ckfree((char *) textPtr);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TileChangedProc
- *
- * Results:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-/* ARGSUSED */
-static void
-TileChangedProc(clientData, tile, itemPtr)
-    ClientData clientData;
-    Tk_Tile tile;
-    Tk_Item *itemPtr;			/* Not used */
-{
-    TkTextRelayoutWindow((TkText *) clientData);
-}
-
 /*
  *----------------------------------------------------------------------
  *
@@ -933,10 +765,6 @@ ConfigureText(interp, textPtr, argc, argv, flags)
     int flags;			/* Flags to pass to Tk_ConfigureWidget. */
 {
     int oldExport = textPtr->exportSelection;
-    XGCValues gcValues;
-    GC new = None;
-    Tk_Tile tile;
-    Pixmap pixmap;
 
     if (Tk_ConfigureWidget(interp, textPtr->tkwin, configSpecs,
 	    argc, argv, (char *) textPtr, flags) != TCL_OK) {
@@ -948,30 +776,24 @@ ConfigureText(interp, textPtr, argc, argv, flags)
      * the geometry and setting the background from a 3-D border.
      */
 
-    tile = textPtr->tile;
-    if (textPtr->state == TK_STATE_DISABLED &&
-	    textPtr->disabledTile != NULL) {
-	tile = textPtr->disabledTile;
+    if ((textPtr->state != tkTextNormalUid)
+	    && (textPtr->state != tkTextDisabledUid)) {
+	Tcl_AppendResult(interp, "bad state value \"", textPtr->state,
+		"\": must be normal or disabled", (char *) NULL);
+	textPtr->state = tkTextNormalUid;
+	return TCL_ERROR;
     }
 
-    Tk_SetTileChangedProc(textPtr->disabledTile, (Tk_TileChangedProc *) NULL,
-		(ClientData) NULL, (Tk_Item *) NULL);
-    Tk_SetTileChangedProc(textPtr->tile, (Tk_TileChangedProc *) NULL,
-		(ClientData) NULL, (Tk_Item *) NULL);
-    Tk_SetTileChangedProc(tile, TileChangedProc,
-		(ClientData) textPtr, (Tk_Item *) NULL);
+    if ((textPtr->wrapMode != tkTextCharUid)
+	    && (textPtr->wrapMode != tkTextNoneUid)
+	    && (textPtr->wrapMode != tkTextWordUid)) {
+	Tcl_AppendResult(interp, "bad wrap mode \"", textPtr->wrapMode,
+		"\": must be char, none, or word", (char *) NULL);
+	textPtr->wrapMode = tkTextCharUid;
+	return TCL_ERROR;
+    }
 
-    if((pixmap = Tk_PixmapOfTile(tile)) != None) {
-	gcValues.fill_style = FillTiled;
-	gcValues.tile = pixmap;
-	new = Tk_GetGC(textPtr->tkwin, GCTile|GCFillStyle, &gcValues);
-    } else {
-	Tk_SetBackgroundFromBorder(textPtr->tkwin, textPtr->border);
-    }
-    if (textPtr->tileGC != None) {
-	Tk_FreeGC(textPtr->display, textPtr->tileGC);
-    }
-    textPtr->tileGC = new;
+    Tk_SetBackgroundFromBorder(textPtr->tkwin, textPtr->border);
 
     /*
      * Don't allow negative spacings.
@@ -1044,9 +866,8 @@ ConfigureText(interp, textPtr, argc, argv, flags)
 	    || (textPtr->selTagPtr->spacing2String != NULL)
 	    || (textPtr->selTagPtr->spacing3String != NULL)
 	    || (textPtr->selTagPtr->tabString != NULL)
-	    || (textPtr->selTagPtr->state != TK_STATE_NULL)
 	    || (textPtr->selTagPtr->underlineString != NULL)
-	    || (textPtr->selTagPtr->wrapMode != TEXT_WRAPMODE_NULL)) {
+	    || (textPtr->selTagPtr->wrapMode != NULL)) {
 	textPtr->selTagPtr->affectsDisplay = 1;
     }
     TkTextRedrawTag(textPtr, (TkTextIndex *) NULL, (TkTextIndex *) NULL,
@@ -1592,7 +1413,7 @@ TextFetchSelection(clientData, offset, buffer, maxBytes)
 		    }
 		}
 	    }
-	    if (segPtr->typePtr == &tkTextCharType && !TkTextIsElided(textPtr, &textPtr->selIndex)) {
+	    if (segPtr->typePtr == &tkTextCharType) {
 		memcpy((VOID *) buffer, (VOID *) (segPtr->body.chars
 			+ offsetInSeg), (size_t) chunkSize);
 		buffer += chunkSize;
@@ -1733,7 +1554,7 @@ TextSearchCmd(textPtr, interp, argc, argv)
     int argc;			/* Number of arguments. */
     char **argv;		/* Argument strings. */
 {
-    int backwards, exact, searchElide, c, i, argsLeft, noCase, leftToScan;
+    int backwards, exact, c, i, argsLeft, noCase, leftToScan;
     size_t length;
     int numLines, startingLine, startingChar, lineNum, firstChar, lastChar;
     int code, matchLength, matchChar, passes, stopLine, searchWholeText;
@@ -1744,7 +1565,6 @@ TextSearchCmd(textPtr, interp, argc, argv)
     Tcl_DString line, patDString;
     TkTextSegment *segPtr;
     TkTextLine *linePtr;
-    TkTextIndex curIndex;
     Tcl_RegExp regexp = NULL;		/* Initialization needed only to
 					 * prevent compiler warning. */
 
@@ -1753,8 +1573,6 @@ TextSearchCmd(textPtr, interp, argc, argv)
      */
 
     exact = 1;
-    searchElide = 0;
-    curIndex.tree = textPtr->tree;
     backwards = 0;
     noCase = 0;
     varName = NULL;
@@ -1768,7 +1586,7 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	    badSwitch:
 	    Tcl_AppendResult(interp, "bad switch \"", arg,
 		    "\": must be -forward, -backward, -exact, -regexp, ",
-		    "-nocase, -count, -hidden, or --", (char *) NULL);
+		    "-nocase, -count, or --", (char *) NULL);
 	    return TCL_ERROR;
 	}
 	c = arg[1];
@@ -1783,16 +1601,12 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	    varName = argv[i];
 	} else if ((c == 'e') && (strncmp(argv[i], "-exact", length) == 0)) {
 	    exact = 1;
-	} else if ((c == 'e') && (strncmp(argv[i], "-elide", length) == 0)) {
-	    searchElide = 1;
 	} else if ((c == 'f') && (strncmp(argv[i], "-forwards", length) == 0)) {
 	    backwards = 0;
 	} else if ((c == 'n') && (strncmp(argv[i], "-nocase", length) == 0)) {
 	    noCase = 1;
 	} else if ((c == 'r') && (strncmp(argv[i], "-regexp", length) == 0)) {
 	    exact = 0;
-	} else if ((c == 'h') && (strncmp(argv[i], "-hidden", length) == 0)) {
-	    searchElide = 1;
 	} else if ((c == '-') && (strncmp(argv[i], "--", length) == 0)) {
 	    i++;
 	    break;
@@ -1888,10 +1702,9 @@ TextSearchCmd(textPtr, interp, argc, argv)
 	 */
 
 	linePtr = TkBTreeFindLine(textPtr->tree, lineNum);
-	curIndex.linePtr = linePtr; curIndex.charIndex = 0;
 	for (segPtr = linePtr->segPtr; segPtr != NULL;
-		curIndex.charIndex += segPtr->size, segPtr = segPtr->nextPtr) {
-	    if (segPtr->typePtr != &tkTextCharType || (!searchElide && TkTextIsElided(textPtr, &curIndex))) {
+		segPtr = segPtr->nextPtr) {
+	    if (segPtr->typePtr != &tkTextCharType) {
 		continue;
 	    }
 	    Tcl_DStringAppend(&line, segPtr->body.chars, segPtr->size);
