@@ -10,7 +10,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkText.h 1.46 96/11/25 11:26:12
+ * RCS: @(#) $Id: tkText.h,v 1.2 1998/09/14 18:23:18 stanton Exp $
  */
 
 #ifndef _TKTEXT
@@ -269,12 +269,6 @@ struct TkTextDispChunk {
  * referred to in other structures.
  */
 
-typedef enum {	TEXT_WRAPMODE_NULL, TEXT_WRAPMODE_NONE,
-		TEXT_WRAPMODE_CHAR, TEXT_WRAPMODE_WORD
-} TkWrapMode;
-
-extern Tk_CustomOption textWrapModeOption;
-
 typedef struct TkTextTag {
     char *name;			/* Name of this tag.  This field is actually
 				 * a pointer to the key from the entry in
@@ -372,16 +366,13 @@ typedef struct TkTextTag {
     int underline;		/* Non-zero means draw underline underneath
 				 * text.  Only valid if underlineString is
 				 * non-NULL. */
-    TkWrapMode wrapMode;	/* How to handle wrap-around for this tag.
-				 * Must be TEXT_WRAPMODE_CHAR,
-				 * TEXT_WRAPMODE_NONE, TEXT_WRAPMODE_WORD,
-				 * or TEXT_WRAPMODE_NULL to use wrapmode for
-				 * whole widget. */
+    Tk_Uid wrapMode;		/* How to handle wrap-around for this tag.
+				 * Must be tkTextCharUid, tkTextNoneUid,
+				 * tkTextWordUid, or NULL to use wrapMode
+				 * for whole widget. */
     int affectsDisplay;		/* Non-zero means that this tag affects the
 				 * way information is displayed on the screen
 				 * (so need to redisplay if tag changes). */
-    Tk_State state;		/* Must be TK_STATE_NULL, TK_STATE_NORMAL,
-				 * TK_STATE_HIDDEN or TK_STATE_DISABLED. */
 } TkTextTag;
 
 #define TK_TAG_AFFECTS_DISPLAY	0x1
@@ -479,8 +470,8 @@ typedef struct TkText {
 				 * image segment doesn't yet have an
 				 * associated image, there is no entry for
 				 * it here. */
-    Tk_State state;		/* Normal, hidden or disabled.  Text is
- 				 * read-only when disabled. */
+    Tk_Uid state;		/* Normal or disabled.  Text is read-only
+				 * when disabled. */
 
     /*
      * Default information for displaying (may be overridden by tags
@@ -522,9 +513,9 @@ typedef struct TkText {
      * Additional information used for displaying:
      */
 
-    TkWrapMode wrapMode;	/* How to handle wrap-around.  Must be
-				 * TEXT_WRAPMODE_CHAR, TEXT_WRAPMODE_NONE, or
-				 * TEXT_WRAPMODE_WORD. */
+    Tk_Uid wrapMode;		/* How to handle wrap-around.  Must be
+				 * tkTextCharUid, tkTextNoneUid, or
+				 * tkTextWordUid. */
     int width, height;		/* Desired dimensions for window, measured
 				 * in characters. */
     int setGrid;		/* Non-zero means pass gridding information
@@ -618,16 +609,6 @@ typedef struct TkText {
 				 * vertical scrollbar when view changes. */
     int flags;			/* Miscellaneous flags;  see below for
 				 * definitions. */
-
-    /*
-     * Additional information, added by the 'dash'-patch
-     */
-
-    Tk_Tile tile;		/* Tile to display in background. */
-    Tk_Tile disabledTile;	/* Tile to display in background if state
-				 * is disabled. */
-    GC tileGC;			/* GC to store background tile. */
-    Tk_TSOffset tsoffset;	/* tile offset */
 } TkText;
 
 /*
@@ -671,7 +652,7 @@ typedef void		Tk_SegLineChangeProc _ANSI_ARGS_((
 typedef int		Tk_SegLayoutProc _ANSI_ARGS_((struct TkText *textPtr,
 			    struct TkTextIndex *indexPtr, TkTextSegment *segPtr,
 			    int offset, int maxX, int maxChars,
-			    int noCharsYet, TkWrapMode wrapMode,
+			    int noCharsYet, Tk_Uid wrapMode,
 			    struct TkTextDispChunk *chunkPtr));
 typedef void		Tk_SegCheckProc _ANSI_ARGS_((TkTextSegment *segPtr,
 			    TkTextLine *linePtr));
@@ -730,10 +711,15 @@ typedef struct Tk_SegType {
 extern int		tkBTreeDebug;
 extern int		tkTextDebug;
 extern Tk_SegType	tkTextCharType;
+extern Tk_Uid		tkTextCharUid;
+extern Tk_Uid		tkTextDisabledUid;
 extern Tk_SegType	tkTextLeftMarkType;
+extern Tk_Uid		tkTextNoneUid;
+extern Tk_Uid 		tkTextNormalUid;
 extern Tk_SegType	tkTextRightMarkType;
 extern Tk_SegType	tkTextToggleOnType;
 extern Tk_SegType	tkTextToggleOffType;
+extern Tk_Uid		tkTextWordUid;
 
 /*
  * Declarations for procedures that are used by the text-related files
@@ -783,7 +769,7 @@ extern int		TkTextCharBbox _ANSI_ARGS_((TkText *textPtr,
 extern int		TkTextCharLayoutProc _ANSI_ARGS_((TkText *textPtr,
 			    TkTextIndex *indexPtr, TkTextSegment *segPtr,
 			    int offset, int maxX, int maxChars, int noBreakYet,
-			    TkWrapMode wrapMode, TkTextDispChunk *chunkPtr));
+			    Tk_Uid wrapMode, TkTextDispChunk *chunkPtr));
 extern void		TkTextCreateDInfo _ANSI_ARGS_((TkText *textPtr));
 extern int		TkTextDLineInfo _ANSI_ARGS_((TkText *textPtr,
 			    TkTextIndex *indexPtr, int *xPtr, int *yPtr,
@@ -815,8 +801,6 @@ extern void		TkTextLostSelection _ANSI_ARGS_((
 extern TkTextIndex *	TkTextMakeIndex _ANSI_ARGS_((TkTextBTree tree,
 			    int lineIndex, int charIndex,
 			    TkTextIndex *indexPtr));
-extern int		TkTextIsElided _ANSI_ARGS_((TkText *textPtr,
-			    TkTextIndex *indexPtr));
 extern int		TkTextMarkCmd _ANSI_ARGS_((TkText *textPtr,
 			    Tcl_Interp *interp, int argc, char **argv));
 extern int		TkTextMarkNameToIndex _ANSI_ARGS_((TkText *textPtr,
