@@ -11,7 +11,7 @@
 #	files by clicking on the file icons or by entering a filename
 #	in the "Filename:" entry.
 #
-# SCCS: @(#) tkfbox.tcl 1.13 97/10/01 14:51:01
+# RCS: @(#) $Id: tkfbox.tcl,v 1.8 1998/11/12 06:22:05 welch Exp $
 #
 # Copyright (c) 1994-1996 Sun Microsystems, Inc.
 #
@@ -822,15 +822,16 @@ proc tkFDialog_Create {w} {
     set data(dirMenu) [tk_optionMenu $f1.menu [format %s(selectPath) $dataName] ""]
     set data(upBtn) [button $f1.up]
     if {![info exists tkPriv(updirImage)]} {
-	set tkPriv(updirImage) [image create bitmap \
-		-data "#define updir_width 28\n#define updir_height 16
-static char updir_bits\[\] = {
+	set tkPriv(updirImage) [image create bitmap -data {
+#define updir_width 28
+#define updir_height 16
+static char updir_bits[] = {
    0x00, 0x00, 0x00, 0x00, 0x80, 0x1f, 0x00, 0x00, 0x40, 0x20, 0x00, 0x00,
    0x20, 0x40, 0x00, 0x00, 0xf0, 0xff, 0xff, 0x01, 0x10, 0x00, 0x00, 0x01,
    0x10, 0x02, 0x00, 0x01, 0x10, 0x07, 0x00, 0x01, 0x90, 0x0f, 0x00, 0x01,
    0x10, 0x02, 0x00, 0x01, 0x10, 0x02, 0x00, 0x01, 0x10, 0x02, 0x00, 0x01,
    0x10, 0xfe, 0x07, 0x01, 0x10, 0x00, 0x00, 0x01, 0x10, 0x00, 0x00, 0x01,
-   0xf0, 0xff, 0xff, 0x01};"]
+   0xf0, 0xff, 0xff, 0x01};}]
     }
     $data(upBtn) config -image $tkPriv(updirImage)
 
@@ -1253,7 +1254,7 @@ proc tkFDialog_ActivateEnt {w} {
     set path [lindex $list 1]
     set file [lindex $list 2]
 
-    case $flag {
+    switch -- $flag {
 	OK {
 	    if {![string compare $file ""]} {
 		# user has entered an existing (sub)directory
