@@ -4,35 +4,40 @@
  *	Provides a version of the Tcl_AppInit procedure for the example shell.
  *
  * Copyright (c) 1993-1994 Lockheed Missle & Space Company, AI Center
- * Copyright (c) 1995-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMacAppInit.c 1.27 96/12/18 13:40:11
+ * RCS: @(#) $Id: tkMacAppInit.c,v 1.5 1999/02/04 20:57:18 stanton Exp $
  */
 
-#include <Windows.h>
 #include <Gestalt.h>
 #include <ToolUtils.h>
 #include <Fonts.h>
 #include <Dialogs.h>
-#include <Memory.h>
 #include <SegLoad.h>
+#include <Traps.h>
+#include <Appearance.h>
 
 #include "tk.h"
 #include "tkInt.h"
 #include "tkMacInt.h"
-#include "tclMacInt.h"
+#include "tclMac.h"
 
 #ifdef TK_TEST
-EXTERN int		Tktest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Tktest_Init _ANSI_ARGS_((Tcl_Interp *interp));
 #endif /* TK_TEST */
 
-typedef int (*TclMacConvertEventPtr) _ANSI_ARGS_((EventRecord *eventPtr));
+#ifdef TCL_TEST
+extern int		Procbodytest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Procbodytest_SafeInit _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		TclObjTest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+extern int		Tcltest_Init _ANSI_ARGS_((Tcl_Interp *interp));
+#endif /* TCL_TEST */
+
 Tcl_Interp *gStdoutInterp = NULL;
 
-void 	TclMacSetEventProc _ANSI_ARGS_((TclMacConvertEventPtr procPtr));
 int 	TkMacConvertEvent _ANSI_ARGS_((EventRecord *eventPtr));
 
 /*
@@ -45,6 +50,14 @@ long			ReadCharsFromConsole _ANSI_ARGS_((char *buff, long n));
 extern char *		__ttyname _ANSI_ARGS_((long fildes));
 short			SIOUXHandleOneEvent _ANSI_ARGS_((EventRecord *event));
 
+/*
+ * Prototypes for functions from the tkConsole.c file.
+ */
+ 
+EXTERN void		TkConsoleCreate _ANSI_ARGS_((void));
+EXTERN int		TkConsoleInit _ANSI_ARGS_((Tcl_Interp *interp));
+EXTERN void		TkConsolePrint _ANSI_ARGS_((Tcl_Interp *interp,
+			    int devId, char *buffer, long size));
 /*
  * Forward declarations for procedures defined later in this file:
  */
@@ -71,9 +84,9 @@ static int		SetupMainInterp _ANSI_ARGS_((Tcl_Interp *interp));
  */
 
 void
-main(argc, argv)
-    int argc;				/* Number of arguments. */
-    char **argv;			/* Array of argument strings. */
+main(
+    int argc,				/* Number of arguments. */
+    char **argv)			/* Array of argument strings. */
 {
     char *newArgv[2];
 
@@ -107,8 +120,8 @@ main(argc, argv)
  */
 
 int
-Tcl_AppInit(interp)
-    Tcl_Interp *interp;		/* Interpreter for application. */
+Tcl_AppInit(
+    Tcl_Interp *interp)		/* Interpreter for application. */
 {
     if (Tcl_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
@@ -116,7 +129,7 @@ Tcl_AppInit(interp)
     if (Tk_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
     }
-    Tcl_StaticPackage(interp, "Tk", Tk_Init, (Tcl_PackageInitProc *) NULL);
+    Tcl_StaticPackage(interp, "Tk", Tk_Init, Tk_SafeInit);
 	
     /*
      * Call the init procedures for included packages.  Each call should
@@ -129,6 +142,22 @@ Tcl_AppInit(interp)
      * where "Mod" is the name of the module.
      */
 
+#ifdef TCL_TEST
+    if (Tcltest_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
+    Tcl_StaticPackage(interp, "Tcltest", Tcltest_Init,
+            (Tcl_PackageInitProc *) NULL);
+    if (TclObjTest_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
+    if (Procbodytest_Init(interp) == TCL_ERROR) {
+	return TCL_ERROR;
+    }
+    Tcl_StaticPackage(interp, "procbodytest", Procbodytest_Init,
+            Procbodytest_SafeInit);
+#endif /* TCL_TEST */
+
 #ifdef TK_TEST
     if (Tktest_Init(interp) == TCL_ERROR) {
 	return TCL_ERROR;
@@ -140,7 +169,7 @@ Tcl_AppInit(interp)
     /*
      * Call Tcl_CreateCommand for application-specific commands, if
      * they weren't already created by the init procedures called above.
-     * Each call would loo like this:
+     * Each call would look like this:
      *
      * Tcl_CreateCommand(interp, "tclName", CFuncCmd, NULL, NULL);
      */
@@ -184,8 +213,34 @@ MacintoshInit()
 {
     int i;
     long result, mask = 0x0700; 		/* mask = system 7.x */
-    
-    InitGraf(&qd.thePort);
+
+#if GENERATING68K && !GENERATINGCFM
+    SetApplLimit(GetApplLimit() - (TK_MAC_68K_STACK_GROWTH));
+#endif
+    MaxApplZone();
+    for (i = 0; i < 4; i++) {
+	(void) MoreMasters();
+    }
+
+    /*
+     * Tk needs us to set the qd pointer it uses.  This is needed
+     * so Tk doesn't have to assume the availablity of the qd global
+     * variable.  Which in turn allows Tk to be used in code resources.
+     */
+    tcl_macQdPtr = &qd;
+
+    /*
+     * If appearance is present, then register Tk as an Appearance client
+     * This means that the mapping from non-Appearance to Appearance cdefs
+     * will be done for Tk regardless of the setting in the Appearance
+     * control panel.  
+     */
+     
+     if (TkMacHaveAppearance()) {
+         RegisterAppearanceClient();
+     }
+
+    InitGraf(&tcl_macQdPtr->thePort);
     InitFonts();
     InitWindows();
     InitMenus();
@@ -196,8 +251,10 @@ MacintoshInit()
      * Make sure we are running on system 7 or higher
      */
      
-    if (((Gestalt(gestaltSystemVersion, &result) != noErr)
-	    || (mask != (result & mask)))) {
+    if ((NGetTrapAddress(_Gestalt, ToolTrap) == 
+    	    NGetTrapAddress(_Unimplemented, ToolTrap))
+    	    || (((Gestalt(gestaltSystemVersion, &result) != noErr)
+	    || (result < mask)))) {
 	panic("Tcl/Tk requires System 7 or higher.");
     }
 
@@ -205,7 +262,7 @@ MacintoshInit()
      * Make sure we have color quick draw 
      * (this means we can't run on 68000 macs)
      */
-    
+     
     if (((Gestalt(gestaltQuickdrawVersion, &result) != noErr)
 	    || (result < gestalt32BitQD13))) {
 	panic("Tk requires Color QuickDraw.");
@@ -215,19 +272,8 @@ MacintoshInit()
     FlushEvents(everyEvent, 0);
     SetEventMask(everyEvent);
 
-    /*
-     * Set up stack & heap sizes
-     */
-    /* TODO: stack size
-       size = StackSpace();
-       SetAppLimit(GetAppLimit() - 8192);
-     */
-    MaxApplZone();
-    for (i = 0; i < 4; i++) {
-	(void) MoreMasters();
-    }
 
-    TclMacSetEventProc(TkMacConvertEvent);
+    Tcl_MacSetEventProc(TkMacConvertEvent);
     TkConsoleCreate();
 
     return TCL_OK;
@@ -253,27 +299,28 @@ MacintoshInit()
  */
 
 static int
-SetupMainInterp(interp)
-    Tcl_Interp *interp;
+SetupMainInterp(
+    Tcl_Interp *interp)
 {
     /*
      * Initialize the console only if we are running as an interactive
      * application.
      */
 
+    TkMacInitAppleEvents(interp);
+    TkMacInitMenus(interp);
+
     if (strcmp(Tcl_GetVar(interp, "tcl_interactive", TCL_GLOBAL_ONLY), "1")
 	    == 0) {
 	if (TkConsoleInit(interp) == TCL_ERROR) {
-	goto error;
+	    goto error;
 	}
     }
 
-    TkMacInitAppleEvents(interp);
-    TkMacInitMenus(interp);
-    
     /*
      * Attach the global interpreter to tk's expected global console
      */
+
     gStdoutInterp = interp;
 
     return TCL_OK;
