@@ -3,12 +3,12 @@
  *
  *	Implements subwindows for the macintosh version of Tk.
  *
- * Copyright (c) 1995-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1995-1997 Sun Microsystems, Inc.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkMacSubwindows.c 1.57 96/10/27 10:26:30
+ * RCS: @(#) $Id: tkMacSubwindows.c,v 1.4 1998/09/14 18:23:39 stanton Exp $
  */
 
 #include "tkInt.h"
@@ -27,81 +27,7 @@ static RgnHandle tmpRgn = NULL;
 
 static void UpdateOffsets _ANSI_ARGS_((TkWindow *winPtr, int deltaX, int deltaY));
 
-extern Tcl_HashTable windowTable;
-void MacMoveWindow _ANSI_ARGS_((WindowRef window, int x, int y));
-
-/*
- *----------------------------------------------------------------------
- *
- * TkMakeWindow --
- *
- *	Creates an X Window (Mac subwindow).
- *
- * Results:
- *	The window id is returned.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-Window
-TkMakeWindow(winPtr, parent)
-    TkWindow *winPtr;
-    Window parent;
-{
-    MacDrawable *macWin;
-    XEvent event;
-
-    /*
-     * Allocate sub window
-     */
-    macWin = (MacDrawable *) ckalloc(sizeof(MacDrawable));
-    if (macWin == NULL) {
-	winPtr->privatePtr = NULL;
-	return None;
-    }
-    macWin->winPtr = winPtr;
-    winPtr->privatePtr = macWin;
-    macWin->clipRgn = NewRgn();
-    macWin->aboveClipRgn = NewRgn();
-    macWin->referenceCount = 0;
-    macWin->flags = TK_CLIP_INVALID;
-    macWin->scrollWinPtr = NULL;
-
-    if (Tk_IsTopLevel(macWin->winPtr)) {
-	macWin->portPtr = (GWorldPtr) NULL;  /* This will be set when we are mapped. */
-	macWin->toplevel = macWin;
-	macWin->xOff = 0;
-	macWin->yOff = 0;
-    } else {
-	macWin->portPtr = NULL;
-	macWin->xOff = winPtr->parentPtr->privatePtr->xOff +
-	    winPtr->parentPtr->changes.border_width +
-	    winPtr->changes.x;
-	macWin->yOff = winPtr->parentPtr->privatePtr->yOff +
-	    winPtr->parentPtr->changes.border_width +
-	    winPtr->changes.y;
-	macWin->toplevel = winPtr->parentPtr->privatePtr->toplevel;
-    }
-
-    macWin->toplevel->referenceCount++;
-    
-    /* 
-     * TODO: need general solution for visibility events.
-     */
-    event.xany.serial = Tk_Display(winPtr)->request;
-    event.xany.send_event = False;
-    event.xany.display = Tk_Display(winPtr);
-	
-    event.xvisibility.type = VisibilityNotify;
-    event.xvisibility.window = (Window) macWin;;
-    event.xvisibility.state = VisibilityUnobscured;
-    Tk_QueueWindowEvent(&event, TCL_QUEUE_TAIL);
-
-    return (Window) macWin;
-}
+void tkMacMoveWindow _ANSI_ARGS_((WindowRef window, int x, int y));
 
 /*
  *----------------------------------------------------------------------
@@ -120,7 +46,9 @@ TkMakeWindow(winPtr, parent)
  */
 
 void 
-XDestroyWindow(Display *display, Window window)
+XDestroyWindow(
+    Display* display,		/* Display. */
+    Window window)		/* Window. */
 {
     MacDrawable *macWin = (MacDrawable *) window;
     GWorldPtr destPort;
@@ -130,38 +58,67 @@ XDestroyWindow(Display *display, Window window)
      * the window we are deleting is being tracked by
      * the grab code.
      */
-    TkMacPointerDeadWindow(macWin->winPtr);
-    TkMacSetScrollbarGrow(macWin->winPtr, false);
-    destPort = TkMacGetDrawablePort(window);
+
+    TkPointerDeadWindow(macWin->winPtr);
     macWin->toplevel->referenceCount--;
     
+    
     if (Tk_IsTopLevel(macWin->winPtr)) {
 	DisposeRgn(macWin->clipRgn);
 	DisposeRgn(macWin->aboveClipRgn);
-	macWin->portPtr = NULL;
-	if (macWin->toplevel->referenceCount == 0) {
-	    ckfree((char *) macWin->toplevel);
-	}
 	
 	/*
 	 * Delete the Mac window and remove it from the windowTable.
 	 * The window could be NULL if the window was never mapped.
+	 * However, we don't do this for embedded windows, they don't
+	 * go in the window list, and they do not own their portPtr's.
 	 */
-	if (destPort != NULL) {
-	    Tcl_DeleteHashEntry(Tcl_FindHashEntry(&windowTable,
-		    (char *) destPort));
-	    DisposeWindow((WindowRef) destPort);
+	 
+	if (!(Tk_IsEmbedded(macWin->winPtr))) {
+            destPort = TkMacGetDrawablePort(window);
+	    if (destPort != NULL) {
+	        TkMacWindowList *listPtr, *prevPtr;
+	    
+	        TkMacUnregisterMacWindow(destPort);
+	        DisposeWindow((WindowRef) destPort);
+	    
+	        for (listPtr = tkMacWindowListPtr, prevPtr = NULL;
+	    	        tkMacWindowListPtr != NULL;
+	    	        prevPtr = listPtr, listPtr = listPtr->nextPtr) {
+	            if (listPtr->winPtr == macWin->winPtr) {
+	                if (prevPtr == NULL) {
+	            	    tkMacWindowListPtr = listPtr->nextPtr;
+	                } else {
+	            	    prevPtr->nextPtr = listPtr->nextPtr;
+	                }
+	                ckfree((char *) listPtr);
+	                break;
+	            }
+	        }
+	    }
+	}
+	
+	macWin->portPtr = NULL;
+	
+	/*
+	 * Delay deletion of a toplevel data structure untill all
+	 * children have been deleted.
+	 */
+	if (macWin->toplevel->referenceCount == 0) {
+	    ckfree((char *) macWin->toplevel);
 	}
     } else {
+        destPort = TkMacGetDrawablePort(window);
 	if (destPort != NULL) {
 	    SetGWorld(destPort, NULL);
-	    InvalRgn(macWin->aboveClipRgn);
+	    TkMacInvalidateWindow(macWin, TK_PARENT_WINDOW);
 	}
 	if (macWin->winPtr->parentPtr != NULL) {
 	    TkMacInvalClipRgns(macWin->winPtr->parentPtr);
 	}
 	DisposeRgn(macWin->clipRgn);
 	DisposeRgn(macWin->aboveClipRgn);
+	
 	if (macWin->toplevel->referenceCount == 0) {
 	    ckfree((char *) macWin->toplevel);
 	}
@@ -187,9 +144,9 @@ XDestroyWindow(Display *display, Window window)
  */
 
 void 
-XMapWindow(display, window)
-    Display *display;
-    Window window;
+XMapWindow(
+    Display* display,		/* Display. */
+    Window window)		/* Window. */
 {
     MacDrawable *macWin = (MacDrawable *) window;
     XEvent event;
@@ -202,7 +159,7 @@ XMapWindow(display, window)
      * window now as this function as well as other X functions assume that
      * the portPtr is valid.
      */
-    if (macWin->toplevel->portPtr == NULL) {
+    if (!TkMacHostToplevelExists(macWin->toplevel->winPtr)) {
 	TkMacMakeRealWindowExist(macWin->toplevel->winPtr);
     }
     destPort = TkMacGetDrawablePort(window);
@@ -210,7 +167,9 @@ XMapWindow(display, window)
     display->request++;
     macWin->winPtr->flags |= TK_MAPPED;
     if (Tk_IsTopLevel(macWin->winPtr)) {
-	ShowWindow((WindowRef) destPort);
+	if (!Tk_IsEmbedded(macWin->winPtr)) {
+	    ShowWindow((WindowRef) destPort);
+	}
 
 	/* 
 	 * We only need to send the MapNotify event
@@ -234,7 +193,7 @@ XMapWindow(display, window)
      */
     SetGWorld(destPort, NULL);
     TkMacUpdateClipRgn(macWin->winPtr);
-    InvalRgn(macWin->aboveClipRgn);
+    TkMacInvalidateWindow(macWin, TK_PARENT_WINDOW);
 }
 
 /*
@@ -255,9 +214,9 @@ XMapWindow(display, window)
  */
 
 void 
-XUnmapWindow(display, window)
-    Display *display;
-    Window window;
+XUnmapWindow(
+    Display* display,		/* Display. */
+    Window window)		/* Window. */
 {
     MacDrawable *macWin = (MacDrawable *) window;
     XEvent event;
@@ -268,7 +227,9 @@ XUnmapWindow(display, window)
     display->request++;
     macWin->winPtr->flags &= ~TK_MAPPED;
     if (Tk_IsTopLevel(macWin->winPtr)) {
-	HideWindow((WindowRef) destPort);
+	if (!Tk_IsEmbedded(macWin->winPtr)) {
+	    HideWindow((WindowRef) destPort);
+	}
 
 	/* 
 	 * We only need to send the UnmapNotify event
@@ -288,7 +249,7 @@ XUnmapWindow(display, window)
 	 * Generate damage for that area of the window.
 	 */
 	SetGWorld(destPort, NULL);
-	InvalRgn(macWin->aboveClipRgn); /* TODO: may not be valid */
+	TkMacInvalidateWindow(macWin, TK_PARENT_WINDOW); /* TODO: may not be valid */
 	TkMacInvalClipRgns(macWin->winPtr->parentPtr);
     }
 }
@@ -311,50 +272,94 @@ XUnmapWindow(display, window)
  */
 
 void 
-XResizeWindow(display, window, width, height)
-    Display *display;
-    Window window; 
-    unsigned int width;
-    unsigned int height;
+XResizeWindow(
+    Display* display,		/* Display. */
+    Window window, 		/* Window. */
+    unsigned int width,
+    unsigned int height)
 {
     MacDrawable *macWin = (MacDrawable *) window;
     GWorldPtr destPort;
 
     destPort = TkMacGetDrawablePort(window);
+    if (destPort == NULL) {
+	return;
+    }
 
     display->request++;
     SetPort((GrafPtr) destPort);
     if (Tk_IsTopLevel(macWin->winPtr)) {
-	/* 
-	 * NOTE: we are not adding the new space to the update
-	 * regoin.  It is currently assumed that Tk will need
-	 * to completely redraw anway.
-	 */
-	SizeWindow((WindowRef) destPort, (short) width, (short) height, false);
-	InvalRgn(macWin->clipRgn);
-	TkMacInvalClipRgns(macWin->winPtr);
+	if (!Tk_IsEmbedded(macWin->winPtr)) {
+	    /* 
+	     * NOTE: we are not adding the new space to the update
+	     * region.  It is currently assumed that Tk will need
+	     * to completely redraw anway.
+	     */
+	    SizeWindow((WindowRef) destPort,
+		    (short) width, (short) height, false);
+	    TkMacInvalidateWindow(macWin, TK_WINDOW_ONLY);
+	    TkMacInvalClipRgns(macWin->winPtr);
+	} else {
+	    int deltaX, deltaY;
+	    
+	    /*
+	     * Find the Parent window -
+	     *    For an embedded window this will be its container.
+	     */
+	    TkWindow *contWinPtr;
+	    
+	    contWinPtr = TkpGetOtherWindow(macWin->winPtr);
+	    
+	    if (contWinPtr != NULL) {
+	        MacDrawable *macParent = contWinPtr->privatePtr;
+
+		TkMacInvalClipRgns(macParent->winPtr);	
+		TkMacInvalidateWindow(macWin, TK_PARENT_WINDOW);
+		
+		deltaX = macParent->xOff +
+		    macWin->winPtr->changes.x - macWin->xOff;
+		deltaY = macParent->yOff +
+		    macWin->winPtr->changes.y - macWin->yOff;
+		
+		UpdateOffsets(macWin->winPtr, deltaX, deltaY);
+	    } else {
+	        /*
+	         * This is the case where we are embedded in
+	         * another app.  At this point, we are assuming that
+	         * the changes.x,y is not maintained, if you need
+		 * the info get it from Tk_GetRootCoords,
+	         * and that the toplevel sits at 0,0 when it is drawn.
+	         */
+		
+		TkMacInvalidateWindow(macWin, TK_PARENT_WINDOW);
+		UpdateOffsets(macWin->winPtr, 0, 0);
+	    }
+	         
+	}   
     } else {
 	/* TODO: update all xOff & yOffs */
-	int deltaX, deltaY;
+	int deltaX, deltaY, parentBorderwidth;
 	MacDrawable *macParent = macWin->winPtr->parentPtr->privatePtr;
-
-	if (!EmptyRgn(macWin->aboveClipRgn)) {
-	    InvalRgn(macWin->aboveClipRgn);
+	
+	if (macParent == NULL) {
+	    return; /* TODO: Probably should be a panic */
 	}
-	TkMacInvalClipRgns(macWin->winPtr->parentPtr);
+	
+	TkMacInvalClipRgns(macParent->winPtr);	
+	TkMacInvalidateWindow(macWin, TK_PARENT_WINDOW);
 
 	deltaX = - macWin->xOff;
 	deltaY = - macWin->yOff;
-	deltaX += macParent->xOff +
-	    macWin->winPtr->parentPtr->changes.border_width +
+
+	parentBorderwidth = macWin->winPtr->parentPtr->changes.border_width;
+	
+	deltaX += macParent->xOff + parentBorderwidth +
 	    macWin->winPtr->changes.x;
-	deltaY += macParent->yOff +
-	    macWin->winPtr->parentPtr->changes.border_width +
+	deltaY += macParent->yOff + parentBorderwidth +
 	    macWin->winPtr->changes.y;
+        
 	UpdateOffsets(macWin->winPtr, deltaX, deltaY);
     }
-    TkGenWMConfigureEvent((Tk_Window) macWin->winPtr, -1, -1, 
-	macWin->winPtr->changes.width, macWin->winPtr->changes.height, TK_SIZE_CHANGED);
 }
 
 /*
@@ -375,58 +380,91 @@ XResizeWindow(display, window, width, height)
  */
 
 void 
-XMoveResizeWindow(Display *display, Window window, 
-	int x, int y,
-	unsigned int width, unsigned int height)
+XMoveResizeWindow(
+    Display* display,		/* Display. */
+    Window window, 		/* Window. */
+    int x, int y,
+    unsigned int width,
+    unsigned int height)
 {	
     MacDrawable *macWin = (MacDrawable *) window;
     GWorldPtr destPort;
 
     destPort = TkMacGetDrawablePort(window);
+    if (destPort == NULL) {
+	return;
+    }
 
     SetPort((GrafPtr) destPort);
-    if (Tk_IsTopLevel(macWin->winPtr)) {
+    if (Tk_IsTopLevel(macWin->winPtr) && !Tk_IsEmbedded(macWin->winPtr)) {	
 	/* 
 	 * NOTE: we are not adding the new space to the update
-	 * regoin.  It is currently assumed that Tk will need
+	 * region.  It is currently assumed that Tk will need
 	 * to completely redraw anway.
 	 */
-	SizeWindow((WindowRef) destPort, (short) width, (short) height, false);
-	MacMoveWindow((WindowRef) destPort, x, y);
-
+	
+	SizeWindow((WindowRef) destPort,
+		(short) width, (short) height, false);
+	tkMacMoveWindow((WindowRef) destPort, x, y);
+	
 	/* TODO: is the following right? */
-	InvalRgn(macWin->clipRgn);
+	TkMacInvalidateWindow(macWin, TK_WINDOW_ONLY);
 	TkMacInvalClipRgns(macWin->winPtr);
-	TkGenWMConfigureEvent((Tk_Window) macWin->winPtr, x, y,
-		width, height, TK_BOTH_CHANGED);
     } else {
-	int deltaX, deltaY;
+	int deltaX, deltaY, parentBorderwidth;
 	Rect bounds;
-	MacDrawable *macParent = macWin->winPtr->parentPtr->privatePtr;
-	if (macParent == NULL) {
-	    return; /* TODO: Probably should be a panic */
+	MacDrawable *macParent;
+	
+        /*
+         * Find the Parent window -
+         *    For an embedded window this will be its container.
+         */
+         
+	if (Tk_IsEmbedded(macWin->winPtr)) {
+	    TkWindow *contWinPtr;
+	    
+	    contWinPtr = TkpGetOtherWindow(macWin->winPtr);
+	    if (contWinPtr == NULL) {
+	            panic("XMoveResizeWindow could not find container");
+	    }
+	    macParent = contWinPtr->privatePtr;
+	    
+	    /*
+	     * NOTE: Here we should handle out of process embedding.
+	     */
+	
+	    
+	} else {
+	    macParent = macWin->winPtr->parentPtr->privatePtr;   
+	    if (macParent == NULL) {
+	        return; /* TODO: Probably should be a panic */
+	    }
 	}
-
-	if (!EmptyRgn(macWin->aboveClipRgn)) {
-	    InvalRgn(macWin->aboveClipRgn);
-	}
-	TkMacInvalClipRgns(macWin->winPtr->parentPtr);
+	        
+	TkMacInvalClipRgns(macParent->winPtr);
+	TkMacInvalidateWindow(macWin, TK_PARENT_WINDOW);
 
 	deltaX = - macWin->xOff;
 	deltaY = - macWin->yOff;
-	deltaX += macParent->xOff +
-	    macWin->winPtr->parentPtr->changes.border_width +
+	
+        /*
+	 * If macWin->winPtr is an embedded window, don't offset by its
+	 *  parent's borderwidth...
+	 */
+	 
+	if (!Tk_IsEmbedded(macWin->winPtr)) {
+	    parentBorderwidth = macWin->winPtr->parentPtr->changes.border_width;
+	} else {
+	    parentBorderwidth = 0;
+	}
+	deltaX += macParent->xOff + parentBorderwidth +
 	    macWin->winPtr->changes.x;
-	deltaY += macParent->yOff +
-	    macWin->winPtr->parentPtr->changes.border_width +
+	deltaY += macParent->yOff + parentBorderwidth +
 	    macWin->winPtr->changes.y;
 		
 	UpdateOffsets(macWin->winPtr, deltaX, deltaY);
 	TkMacWinBounds(macWin->winPtr, &bounds);
 	InvalRect(&bounds);
-	TkGenWMConfigureEvent((Tk_Window) macWin->winPtr, 
-	    macWin->winPtr->changes.x, macWin->winPtr->changes.y, 
-	    width, height, TK_BOTH_CHANGED);
     }
 }
 
@@ -448,58 +486,86 @@ XMoveResizeWindow(Display *display, Window window,
  */
 
 void 
-XMoveWindow(display, window, x, y)
-    Display* display;
-    Window window;
-    int x;
-    int y;
+XMoveWindow(
+    Display* display,		/* Display. */
+    Window window,		/* Window. */
+    int x,
+    int y)
 {
     MacDrawable *macWin = (MacDrawable *) window;
     GWorldPtr destPort;
 
     destPort = TkMacGetDrawablePort(window);
+    if (destPort == NULL) {
+	return;
+    }
 
     SetPort((GrafPtr) destPort);
-    if (Tk_IsTopLevel(macWin->winPtr)) {
+    if (Tk_IsTopLevel(macWin->winPtr) && !Tk_IsEmbedded(macWin->winPtr)) {
 	/* 
 	 * NOTE: we are not adding the new space to the update
-	 * regoin.  It is currently assumed that Tk will need
+	 * region.  It is currently assumed that Tk will need
 	 * to completely redraw anway.
 	 */
-	MacMoveWindow((WindowRef) destPort, x, y);
+	tkMacMoveWindow((WindowRef) destPort, x, y);
 
 	/* TODO: is the following right? */
-	InvalRgn(macWin->clipRgn);
+	TkMacInvalidateWindow(macWin, TK_WINDOW_ONLY);
 	TkMacInvalClipRgns(macWin->winPtr);
-	TkGenWMConfigureEvent((Tk_Window) macWin->winPtr, x, y,
-		-1, -1, TK_LOCATION_CHANGED);
     } else {
-	int deltaX, deltaY;
+	int deltaX, deltaY, parentBorderwidth;
 	Rect bounds;
-	MacDrawable *macParent = macWin->winPtr->parentPtr->privatePtr;
-	if (macParent == NULL) {
-	    return; /* TODO: Probably should be a panic */
+	MacDrawable *macParent;
+	
+        /*
+         * Find the Parent window -
+         * For an embedded window this will be its container.
+         */
+         
+	if (Tk_IsEmbedded(macWin->winPtr)) {
+	    TkWindow *contWinPtr;
+	    
+	    contWinPtr = TkpGetOtherWindow(macWin->winPtr);
+	    if (contWinPtr == NULL) {
+	            panic("XMoveWindow could not find container");
+	    }
+	    macParent = contWinPtr->privatePtr;
+	    
+	    /*
+	     * NOTE: Here we should handle out of process embedding.
+	     */
+		    
+	} else {
+	    macParent = macWin->winPtr->parentPtr->privatePtr;   
+	    if (macParent == NULL) {
+	        return; /* TODO: Probably should be a panic */
+	    }
 	}
 
-	if (!EmptyRgn(macWin->aboveClipRgn)) {
-	    InvalRgn(macWin->aboveClipRgn);
-	}
-	TkMacInvalClipRgns(macWin->winPtr->parentPtr);
+	TkMacInvalClipRgns(macParent->winPtr);
+	TkMacInvalidateWindow(macWin, TK_PARENT_WINDOW);
 
 	deltaX = - macWin->xOff;
 	deltaY = - macWin->yOff;
-	deltaX += macParent->xOff +
-	    macWin->winPtr->parentPtr->changes.border_width +
+	
+        /*
+	 * If macWin->winPtr is an embedded window, don't offset by its
+	 *  parent's borderwidth...
+	 */
+	 
+	if (!Tk_IsEmbedded(macWin->winPtr)) {
+	    parentBorderwidth = macWin->winPtr->parentPtr->changes.border_width;
+	} else {
+	    parentBorderwidth = 0;
+	}
+	deltaX += macParent->xOff + parentBorderwidth +
 	    macWin->winPtr->changes.x;
-	deltaY += macParent->yOff +
-	    macWin->winPtr->parentPtr->changes.border_width +
+	deltaY += macParent->yOff + parentBorderwidth +
 	    macWin->winPtr->changes.y;
 		
 	UpdateOffsets(macWin->winPtr, deltaX, deltaY);
 	TkMacWinBounds(macWin->winPtr, &bounds);
 	InvalRect(&bounds);
-	TkGenWMConfigureEvent((Tk_Window) macWin->winPtr, 
-	    macWin->winPtr->changes.x, macWin->winPtr->changes.y, -1, -1, TK_LOCATION_CHANGED);
     }
 }
 
@@ -520,14 +586,14 @@ XMoveWindow(display, window, x, y)
  */
 
 void 
-XRaiseWindow(display, window)
-    Display* display;
-    Window window;
+XRaiseWindow(
+    Display* display,		/* Display. */
+    Window window)		/* Window. */
 {
     MacDrawable *macWin = (MacDrawable *) window;
     
     display->request++;
-    if (Tk_IsTopLevel(macWin->winPtr)) {
+    if (Tk_IsTopLevel(macWin->winPtr) && !Tk_IsEmbedded(macWin->winPtr)) {
 	TkWmRestackToplevel(macWin->winPtr, Above, NULL);
     } else {
     	/* TODO: this should generate damage */
@@ -554,11 +620,11 @@ XRaiseWindow(display, window)
  */
 
 void
-XConfigureWindow(display, w, value_mask, values)
-    Display* display;
-    Window w;
-    unsigned int value_mask;
-    XWindowChanges* values;
+XConfigureWindow(
+    Display* display,		/* Display. */
+    Window w,			/* Window. */
+    unsigned int value_mask,
+    XWindowChanges* values)
 {
     MacDrawable *macWin = (MacDrawable *) w;
     TkWindow *winPtr = macWin->winPtr;
@@ -586,10 +652,12 @@ XConfigureWindow(display, w, value_mask, values)
 	GWorldPtr destPort;
 	
 	destPort = TkMacGetDrawablePort(w);
-	SetPort((GrafPtr) destPort);
-	TkMacInvalClipRgns(winPtr->parentPtr);
-	TkMacWinBounds(winPtr, &bounds);
-	InvalRect(&bounds);
+	if (destPort != NULL) {
+	    SetPort((GrafPtr) destPort);
+	    TkMacInvalClipRgns(winPtr->parentPtr);
+	    TkMacWinBounds(winPtr, &bounds);
+	    InvalRect(&bounds);
+	}
     } 
 
     /* TkGenWMMoveRequestEvent(macWin->winPtr, 
@@ -616,8 +684,8 @@ XConfigureWindow(display, w, value_mask, values)
  */
 
 void
-TkMacUpdateClipRgn(winPtr)
-    TkWindow *winPtr;
+TkMacUpdateClipRgn(
+    TkWindow *winPtr)
 {
     RgnHandle rgn;
     int x, y;
@@ -634,21 +702,29 @@ TkMacUpdateClipRgn(winPtr)
 	}
 	
 	/* 
-	 * Start with a region defined by the window bounds.
+	 * Start with a region defined by the window bounds.  
 	 */
-	x = winPtr->privatePtr->xOff;
-	y = winPtr->privatePtr->yOff;
-	SetRectRgn(rgn, (short) x, (short) y,
-		(short) (winPtr->changes.width  + x), 
-		(short) (winPtr->changes.height + y));
 
+        x = winPtr->privatePtr->xOff;
+        y = winPtr->privatePtr->yOff;
+        SetRectRgn(rgn, (short) x, (short) y,
+	    (short) (winPtr->changes.width  + x), 
+	    (short) (winPtr->changes.height + y));
+	    
 	/* 
 	 * Clip away the area of any windows that may obscure this
-	 * window.  First, clip to the parents visable clip region.
+	 * window.  
+	 * For a non-toplevel window, first, clip to the parents visable
+	 * clip region.
 	 * Second, clip away any siblings that are higher in the
 	 * stacking order.
+	 * For an embedded toplevel, just clip to the container's visible
+	 * clip region.  Remember, we only allow one contained window 
+	 * in a frame, and don't support any other widgets in the frame either.
+	 * This is not currently enforced, however.
 	 */
-	if (!Tk_IsTopLevel(winPtr)) {
+	
+	if (!Tk_IsTopLevel(winPtr)) { 
 	    TkMacUpdateClipRgn(winPtr->parentPtr);
 	    SectRgn(rgn, 
 		    winPtr->parentPtr->privatePtr->aboveClipRgn, rgn);
@@ -668,12 +744,33 @@ TkMacUpdateClipRgn(winPtr)
 							  
 		win2Ptr = win2Ptr->nextPtr;
 	    }
+	} else if (Tk_IsEmbedded(winPtr)) {
+            TkWindow *contWinPtr;
+        
+	    contWinPtr = TkpGetOtherWindow(winPtr);
+    	     
+    	    if (contWinPtr != NULL) {
+ 	        TkMacUpdateClipRgn(contWinPtr);
+	        SectRgn(rgn, 
+		        contWinPtr->privatePtr->aboveClipRgn, rgn);
+   	    } else if (gMacEmbedHandler != NULL) {
+   	        gMacEmbedHandler->getClipProc((Tk_Window) winPtr, tmpRgn);
+   	        SectRgn(rgn, tmpRgn, rgn);
+   	    }
+	    
+	    /*
+	     * NOTE: Here we should handle out of process embedding.
+	     */
+		    
 	}
-		
+	
 	/* 
 	 * The final clip region is the aboveClip region (or visable
 	 * region) minus all the children of this window.
+	 * Alternatively, if the window is a container, we must also 
+	 * subtract the region of the embedded window.
 	 */
+	 
 	rgn = winPtr->privatePtr->clipRgn;
 	CopyRgn(winPtr->privatePtr->aboveClipRgn, rgn);
 		
@@ -692,6 +789,25 @@ TkMacUpdateClipRgn(winPtr)
 							  
 	    win2Ptr = win2Ptr->nextPtr;
 	}
+	
+	if (Tk_IsContainer(winPtr)) {
+	    win2Ptr = TkpGetOtherWindow(winPtr);
+	    if (win2Ptr != NULL) {
+		if (Tk_IsMapped(win2Ptr)) {
+		    x = win2Ptr->privatePtr->xOff;
+		    y = win2Ptr->privatePtr->yOff;
+		    SetRectRgn(tmpRgn, (short) x, (short) y,
+			    (short) (win2Ptr->changes.width  + x), 
+			    (short) (win2Ptr->changes.height + y));
+		    DiffRgn(rgn, tmpRgn, rgn);
+		}
+	    } 
+	    
+	    /*
+	     * NOTE: Here we should handle out of process embedding.
+	     */
+		    
+	}
 		
 	winPtr->privatePtr->flags &= ~TK_CLIP_INVALID;
     }
@@ -715,8 +831,8 @@ TkMacUpdateClipRgn(winPtr)
  */
 
 RgnHandle
-TkMacVisableClipRgn(winPtr)
-    TkWindow *winPtr;
+TkMacVisableClipRgn(
+    TkWindow *winPtr)
 {
     if (winPtr->privatePtr->flags & TK_CLIP_INVALID) {
 	TkMacUpdateClipRgn(winPtr);
@@ -725,6 +841,39 @@ TkMacVisableClipRgn(winPtr)
     return winPtr->privatePtr->clipRgn;
 }
 
+/*
+ *----------------------------------------------------------------------
+ *
+ * TkMacInvalidateWindow --
+ *
+ *	This function makes the window as invalid will generate damage
+ *	for the window.
+ *
+ * Results:
+ *	None.
+ *
+ * Side effects:
+ *	Damage is created.
+ *
+ *----------------------------------------------------------------------
+ */
+
+void
+TkMacInvalidateWindow(
+    MacDrawable *macWin,        /* Make window that's causing damage. */
+    int flag)			/* Should be TK_WINDOW_ONLY or
+				 * TK_PARENT_WINDOW */
+{
+    
+    if (flag == TK_WINDOW_ONLY) {
+	InvalRgn(macWin->clipRgn);
+    } else {
+	if (!EmptyRgn(macWin->aboveClipRgn)) {
+	    InvalRgn(macWin->aboveClipRgn);
+	}
+    }
+}
+
 /*
  *----------------------------------------------------------------------
  *
@@ -742,16 +891,62 @@ TkMacVisableClipRgn(winPtr)
  */
 
 GWorldPtr
-TkMacGetDrawablePort(drawable)
-    Drawable drawable;
+TkMacGetDrawablePort(
+    Drawable drawable)
 {
     MacDrawable *macWin = (MacDrawable *) drawable;
+    GWorldPtr resultPort = NULL;
     
+    if (macWin == NULL) {
+        return NULL;
+    }
+    
+    /*
+     * This is NULL for off-screen pixmaps.  Then the portPtr
+     * always points to the off-screen port, and we don't
+     * have to worry about containment
+     */
+     
     if (macWin->clipRgn == NULL) {
 	return macWin->portPtr;
     }
     
-    return macWin->toplevel->portPtr;
+    /*
+     * If the Drawable is in an embedded window, use the Port of its container.
+     *  
+     * TRICKY POINT: we can have cases when a toplevel is being destroyed
+     * where the winPtr for the toplevel has been freed, but the children 
+     * are not all the way destroyed.  The children will call this function
+     * as they are being destroyed, but Tk_IsEmbedded will return garbage.
+     * So we check the copy of the TK_EMBEDDED flag we put into the 
+     * toplevel's macWin flags.
+     */
+    
+    if (!(macWin->toplevel->flags & TK_EMBEDDED)) {
+        return macWin->toplevel->portPtr;
+    } else {
+    	TkWindow *contWinPtr;
+
+	contWinPtr = TkpGetOtherWindow(macWin->toplevel->winPtr);
+	
+    	if (contWinPtr != NULL) {
+    	    resultPort = TkMacGetDrawablePort((Drawable) contWinPtr->privatePtr);
+    	} else if (gMacEmbedHandler != NULL) {
+	    resultPort = gMacEmbedHandler->getPortProc(
+                    (Tk_Window) macWin->winPtr);
+    	} 
+	
+	if (resultPort == NULL) {
+    	    panic("TkMacGetDrawablePort couldn't find container");
+    	    return NULL;
+    	}	
+	    
+	/*
+	 * NOTE: Here we should handle out of process embedding.
+	 */
+		    
+    }
+    return resultPort;
 }
 
 /*
@@ -775,8 +970,8 @@ TkMacGetDrawablePort(drawable)
  */
 
 void
-TkMacInvalClipRgns(winPtr)
-    TkWindow *winPtr;
+TkMacInvalClipRgns(
+    TkWindow *winPtr)
 {
     TkWindow *childPtr;
 	
@@ -792,7 +987,7 @@ TkMacInvalClipRgns(winPtr)
 	
     /* 
      * Invalidate clip regions for all children & 
-     * thier decendants - unless the child is a toplevel.
+     * their decendants - unless the child is a toplevel.
      */
     childPtr = winPtr->childList;
     while (childPtr != NULL) {
@@ -801,6 +996,23 @@ TkMacInvalClipRgns(winPtr)
 	}
 	childPtr = childPtr->nextPtr;
     }
+    
+    /*
+     * Also, if the window is a container, mark its embedded window
+     */
+     
+    if (Tk_IsContainer(winPtr)) {
+	childPtr = TkpGetOtherWindow(winPtr);
+
+	if (childPtr != NULL && Tk_IsMapped(childPtr)) {
+	    TkMacInvalClipRgns(childPtr);
+	}
+	
+	/*
+	 * NOTE: Here we should handle out of process embedding.
+	 */
+		    	
+    }     	    
 }
 
 /*
@@ -823,9 +1035,9 @@ TkMacInvalClipRgns(winPtr)
  */
 
 void
-TkMacWinBounds(winPtr, bounds)
-    TkWindow *winPtr;
-    Rect *bounds;
+TkMacWinBounds(
+    TkWindow *winPtr,
+    Rect *bounds)
 {
     bounds->left = (short) winPtr->privatePtr->xOff;
     bounds->top = (short) winPtr->privatePtr->yOff;
@@ -838,7 +1050,7 @@ TkMacWinBounds(winPtr, bounds)
 /*
  *----------------------------------------------------------------------
  *
- * MacMoveWindow --
+ * tkMacMoveWindow --
  *
  *	A replacement for the Macintosh MoveWindow function.  This
  *	function adjusts the inputs to MoveWindow to offset the root of 
@@ -855,7 +1067,10 @@ TkMacWinBounds(winPtr, bounds)
  */
 
 void 
-MacMoveWindow(WindowRef window, int x, int y)
+tkMacMoveWindow(
+    WindowRef window,
+    int x,
+    int y)
 {
     int xOffset, yOffset;
 
@@ -883,9 +1098,10 @@ MacMoveWindow(WindowRef window, int x, int y)
  */
 
 static void
-UpdateOffsets(winPtr, deltaX, deltaY)
-    TkWindow *winPtr;
-    int deltaX, deltaY;
+UpdateOffsets(
+    TkWindow *winPtr,
+    int deltaX,
+    int deltaY)
 {
     TkWindow *childPtr;
 
@@ -909,6 +1125,18 @@ UpdateOffsets(winPtr, deltaX, deltaY)
 	}
 	childPtr = childPtr->nextPtr;
     }
+    
+    if (Tk_IsContainer(winPtr)) {
+	childPtr = TkpGetOtherWindow(winPtr);
+	if (childPtr != NULL) {
+	    UpdateOffsets(childPtr,deltaX,deltaY);
+	}
+	    
+	/*
+	 * NOTE: Here we should handle out of process embedding.
+	 */
+		    
+    }
 }
 
 /*
@@ -928,11 +1156,12 @@ UpdateOffsets(winPtr, deltaX, deltaY)
  */
 
 Pixmap
-Tk_GetPixmap(display, d, width, height, depth)
-    Display *display;		/* Display for new pixmap. */
-    Drawable d;			/* Drawable where pixmap will be used. */
-    unsigned int width, height;	/* Dimensions of pixmap. */
-    unsigned int depth;		/* Bits per pixel for pixmap. */
+Tk_GetPixmap(
+    Display *display,	/* Display for new pixmap (can be null). */
+    Drawable d,		/* Drawable where pixmap will be used (ignored). */
+    int width,		/* Dimensions of pixmap. */
+    int height,
+    int depth)		/* Bits per pixel for pixmap. */
 {
     QDErr err;
     GWorldPtr gWorld;
@@ -940,7 +1169,9 @@ Tk_GetPixmap(display, d, width, height, depth)
     MacDrawable *macPix;
     PixMapHandle pixels;
     
-    display->request++;
+    if (display != NULL) {
+	display->request++;
+    }
     macPix = (MacDrawable *) ckalloc(sizeof(MacDrawable));
     macPix->winPtr = NULL;
     macPix->xOff = 0;
@@ -950,20 +1181,22 @@ Tk_GetPixmap(display, d, width, height, depth)
     macPix->referenceCount = 0;
     macPix->toplevel = NULL;
     macPix->flags = 0;
-    macPix->scrollWinPtr = NULL;
 
     bounds.top = bounds.left = 0;
     bounds.right = (short) width;
     bounds.bottom = (short) height;
-    
+    if (depth != 1) {
+	depth = 0;
+    }
+
     /*
      * Allocate memory for the off screen pixmap.  If we fail
      * try again from system memory.  Eventually, we may have
      * to panic.
      */
-    err = NewGWorld(&gWorld, 0, &bounds, NULL, NULL, 0);
+    err = NewGWorld(&gWorld, depth, &bounds, NULL, NULL, 0);
     if (err != noErr) {
-	err = NewGWorld(&gWorld, 0, &bounds, NULL, NULL, useTempMem);
+	err = NewGWorld(&gWorld, depth, &bounds, NULL, NULL, useTempMem);
     }
     if (err != noErr) {
         panic("Out of memory: NewGWorld failed in Tk_GetPixmap");
@@ -996,9 +1229,9 @@ Tk_GetPixmap(display, d, width, height, depth)
  */
 
 void 
-Tk_FreePixmap(display, pixmap)
-    Display *display;		/* Display. */
-    Pixmap pixmap;     		/* Pixmap to destroy */
+Tk_FreePixmap(
+    Display *display,		/* Display. */
+    Pixmap pixmap)     		/* Pixmap to destroy */
 {
     MacDrawable *macPix = (MacDrawable *) pixmap;
     PixMapHandle pixels;
@@ -1010,59 +1243,3 @@ Tk_FreePixmap(display, pixmap)
     ckfree((char *) macPix);
 }
 
-/*
- *----------------------------------------------------------------------
- *
- * TkMacSetScrollbarGrow --
- *
- *	Sets a flag for a toplevel window indicating that the passed
- *	Tk scrollbar window will display the grow region for the 
- *	toplevel window.
- *
- * Results:
- *	None.
- *
- * Side effects:
- *	A flag is set int windows toplevel parent.
- *
- *----------------------------------------------------------------------
- */
-
-void 
-TkMacSetScrollbarGrow(winPtr, flag)
-    TkWindow *winPtr;		/* Tk scrollbar window. */
-    int flag;			/* Boolean value true or false. */
-{
-    if (flag) {
-	winPtr->privatePtr->toplevel->flags |= TK_SCROLLBAR_GROW;
-	winPtr->privatePtr->toplevel->scrollWinPtr = winPtr;
-    } else if (winPtr->privatePtr->toplevel->scrollWinPtr == winPtr) {
-	winPtr->privatePtr->toplevel->flags &= ~TK_SCROLLBAR_GROW;
-	winPtr->privatePtr->toplevel->scrollWinPtr = NULL;	
-    }
-}
-
-/*
- *----------------------------------------------------------------------
- *
- * TkMacGetScrollbarGrowWindow --
- *
- *	Tests to see if a given window's toplevel window contains a
- *	scrollbar that will draw the GrowIcon for the window.
- *
- * Results:
- *	Boolean value.
- *
- * Side effects:
- *	None.
- *
- *----------------------------------------------------------------------
- */
-
-TkWindow * 
-TkMacGetScrollbarGrowWindow(winPtr)
-    TkWindow *winPtr;	/* Tk window. */
-{
-    if (winPtr == NULL) return NULL;
-    return winPtr->privatePtr->toplevel->scrollWinPtr;
-}
