@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkUnixMenu.c 1.76 97/11/05 09:08:22
+ * RCS: @(#) $Id: tkUnixMenu.c,v 1.2 1998/09/14 18:23:57 stanton Exp $
  */
 
 #include "tkPort.h"
@@ -416,7 +416,7 @@ DrawMenuEntryBackground(menuPtr, mePtr, d, activeBorder, bgBorder, x, y,
     int width;			/* Width of entry rect */
     int height;			/* Height of entry rect */
 {
-    if (mePtr->state == TK_STATE_ACTIVE) {
+    if (mePtr->state == tkActiveUid) {
 	int relief;
     	bgBorder = activeBorder;
 
@@ -722,7 +722,7 @@ DrawMenuEntryLabel(
     	}
     }
 
-    if (mePtr->state == TK_STATE_DISABLED) {
+    if (mePtr->state == tkDisabledUid) {
 	if (menuPtr->disabledFg == NULL) {
 	    XFillRectangle(menuPtr->display, d, menuPtr->disabledGC, x, y,
 		    (unsigned) width, (unsigned) height);
@@ -1235,7 +1235,7 @@ TkpDrawMenuEntry(mePtr, d, tkfont, menuMetricsPtr, x, y, width, height,
      * Choose the gc for drawing the foreground part of the entry.
      */
 
-    if ((mePtr->state == TK_STATE_ACTIVE)
+    if ((mePtr->state == tkActiveUid)
 	    && !strictMotif) {
 	gc = mePtr->activeGC;
 	if (gc == NULL) {
@@ -1250,14 +1250,14 @@ TkpDrawMenuEntry(mePtr, d, tkfont, menuMetricsPtr, x, y, width, height,
     		cascadeEntryPtr = cascadeEntryPtr->nextCascadePtr) {
     	    if (strcmp(cascadeEntryPtr->name, 
     	    	    Tk_PathName(menuPtr->tkwin)) == 0) {
-    	    	if (cascadeEntryPtr->state == TK_STATE_DISABLED) {
+    	    	if (cascadeEntryPtr->state == tkDisabledUid) {
     	    	    parentDisabled = 1;
     	    	}
     	    	break;
     	    }
     	}
 
-	if (((parentDisabled || (mePtr->state == TK_STATE_DISABLED)))
+	if (((parentDisabled || (mePtr->state == tkDisabledUid)))
 		&& (menuPtr->disabledFg != NULL)) {
 	    gc = mePtr->disabledGC;
 	    if (gc == NULL) {
