@@ -6,17 +6,20 @@
  *	#includes for system include files and a few other things.
  *
  * Copyright (c) 1991-1993 The Regents of the University of California.
- * Copyright (c) 1994-1995 Sun Microsystems, Inc.
+ * Copyright (c) 1994-1996 Sun Microsystems, Inc.
+ * Copyright (c) 1998 by Scriptics Corporation.
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkUnixPort.h 1.19 96/10/02 15:40:33
+ * RCS: @(#) $Id: tkUnixPort.h,v 1.3 1998/09/30 19:01:22 rjohnson Exp $
  */
 
 #ifndef _UNIXPORT
 #define _UNIXPORT
 
+#define __UNIX__ 1
+
 /*
  * Macro to use instead of "void" for arguments that must have
  * type "void *" in ANSI C;  maps them to type "char *" in
@@ -50,10 +53,23 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/file.h>
+#ifdef HAVE_SYS_SELECT_H
+#   include <sys/select.h>
+#endif
 #include <sys/stat.h>
 #ifndef _TCL
 #   include <tcl.h>
 #endif
+#if TIME_WITH_SYS_TIME
+#   include <sys/time.h>
+#   include <time.h>
+#else
+#   if HAVE_SYS_TIME_H
+#       include <sys/time.h>
+#   else
+#       include <time.h>
+#   endif
+#endif
 #ifdef HAVE_UNISTD_H
 #   include <unistd.h>
 #else
@@ -67,6 +83,43 @@
 #include <X11/Xresource.h>
 #include <X11/Xutil.h>
 
+/*
+ * The following macro defines the type of the mask arguments to
+ * select:
+ */
+
+#ifndef NO_FD_SET
+#   define SELECT_MASK fd_set
+#else
+#   ifndef _AIX
+	typedef long fd_mask;
+#   endif
+#   if defined(_IBMR2)
+#	define SELECT_MASK void
+#   else
+#	define SELECT_MASK int
+#   endif
+#endif
+
+/*
+ * The following macro defines the number of fd_masks in an fd_set:
+ */
+
+#ifndef FD_SETSIZE
+#   ifdef OPEN_MAX
+#	define FD_SETSIZE OPEN_MAX
+#   else
+#	define FD_SETSIZE 256
+#   endif
+#endif
+#if !defined(howmany)
+#   define howmany(x, y) (((x)+((y)-1))/(y))
+#endif
+#ifndef NFDBITS
+#   define NFDBITS NBBY*sizeof(fd_mask)
+#endif
+#define MASK_SIZE howmany(FD_SETSIZE, NFDBITS)
+
 /*
  * Not all systems declare the errno variable in errno.h. so this
  * file does it explicitly.
@@ -105,12 +158,6 @@ extern int errno;
 	XPutImage(display, pixels, gc, image, destx, desty, srcx, \
 	srcy, width, height);
 
-/*
- * The following Tk functions are implemented as macros under Windows.
- */
-
-#define TkGetNativeProlog(interp) TkGetProlog(interp)
-
 /*
  * Supply macros for seek offsets, if they're not already provided by
  * an include file.
@@ -133,13 +180,51 @@ extern int errno;
  * in any other header file.
  */
 
-extern void		panic();
+extern void		panic _ANSI_ARGS_(TCL_VARARGS(char *, string));
 
 /*
- * These functions do nothing under Unix, so we just eliminate calls them.
+ * These functions do nothing under Unix, so we just eliminate calls to them.
  */
 
+#define TkpDestroyButton(butPtr) {}
 #define TkSelUpdateClipboard(a,b) {}
 #define TkSetPixmapColormap(p,c) {}
 
+/*
+ * These calls implement native bitmaps which are not supported under
+ * UNIX.  The macros eliminate the calls.
+ */
+
+#define TkpDefineNativeBitmaps()
+#define TkpCreateNativeBitmap(display, source) None
+#define TkpGetNativeAppBitmap(display, name, w, h) None
+
+/*
+ * This macro stores a representation of the window handle in a string.
+ */
+
+#define TkpPrintWindowId(buf,w) \
+	sprintf((buf), "0x%x", (unsigned int) (w))
+	    
+/*
+ * TkpScanWindowId is just an alias for Tcl_GetInt on Unix.
+ */
+
+#define TkpScanWindowId(i,s,wp) \
+	Tcl_GetInt((i),(s),(wp))
+	    
+/*
+ * This macro indicates that entry and text widgets should display
+ * the selection highlight regardless of which window has the focus.
+ */
+
+#define ALWAYS_SHOW_SELECTION
+
+/*
+ * The following declaration is used to get access to a private Tcl interface
+ * that is needed for portability reasons.
+ */
+
+EXTERN void		TclpGetTime _ANSI_ARGS_((Tcl_Time *time));
+
 #endif /* _UNIXPORT */
