@@ -8,7 +8,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * SCCS: @(#) tkWinDialog.c 1.9 97/07/30 17:25:53
+ * RCS: @(#) $Id: tkWinDialog.c,v 1.2 1998/09/14 18:23:59 stanton Exp $
  *
  */
  
@@ -268,6 +268,13 @@ Tk_ChooseColorCmd(clientData, interp, argc, argv)
     winCode = ChooseColor(&chooseColor);
     (void) Tcl_SetServiceMode(oldMode);
 
+    /*
+     * Clear the interp result since anything may have happened during the
+     * modal loop.
+     */
+
+    Tcl_ResetResult(interp);
+
     /*
      * 3. Process the result of the dialog
      */
@@ -461,6 +468,13 @@ GetFileName(clientData, interp, argc, argv, isOpen)
     SetCurrentDirectory(buffer);
     (void) Tcl_SetServiceMode(oldMode);
 
+    /*
+     * Clear the interp result since anything may have happened during the
+     * modal loop.
+     */
+
+    Tcl_ResetResult(interp);
+
     if (ofnPtr->lpstrInitialDir != NULL) {
 	ckfree((char*) ofnPtr->lpstrInitialDir);
     }
